@@ -3,6 +3,7 @@
  *
  * Copyright (C) 1997-2001 Marko Macek
  */
+#include <algorithm>
 #include "config.h"
 #include "ypaint.h"
 #include "yicon.h"
@@ -12,7 +13,6 @@
 #include "ypointer.h"
 #include "ywordexp.h"
 #include "ascii.h"
-#include <algorithm>
 #include <fnmatch.h>
 #include <dirent.h>
 #include "intl.h"
@@ -377,7 +377,8 @@ public:
         // now test the system icons folders specified by user or defaults
         csmart copy(newstr(iconPath));
         for (tokens folder(copy, ":"); folder; ++folder) {
-            probeIconFolder(folder.token(), false);
+            upath path(folder.token());
+            probeIconFolder(path.expand(), false);
         }
 
         dedupTestPath.clear();
@@ -392,7 +393,7 @@ public:
 
         // for compaction reasons, the lambdas return true on success,
         // but the success is only found in _this_ lambda only,
-        // and this is the only one which touches `result`!
+        // and this is the only one that touches `result`!
         auto checkFile = [&](upath path) {
             return path.fileExists() ? (res = path, true) : false;
         };
@@ -518,7 +519,7 @@ ref<YImage> YIcon::loadIcon(unsigned size) {
         }
 
     }
-    // if the image data which was found in the expected file does not really
+    // if the image data that was found in the expected file does not really
     // match the filename, scale the data to fit
     if (icon != null) {
         if (size != icon->width() || size != icon->height()) {
